@@ -42,7 +42,7 @@
  * @author Akinobu LEE
  * @date   Tue Feb 22 17:00:45 2005
  *
- * $Revision: 1.6 $
+ * $Revision: 1.8 $
  * 
  */
 /*
@@ -295,7 +295,7 @@ trace_backptr(WORD_ID wordseq_rt[MAXSEQNUM], int *rt_wordlen, TRELLIS_ATOM *atom
   int wordlen = 0;		/* word length of best sentence hypothesis */
   TRELLIS_ATOM *tretmp;
   LOGPROB langscore = 0.0;
-  static WORD_ID wordseq[MAXSEQNUM];	/* temporal: in reverse order */
+  WORD_ID wordseq[MAXSEQNUM];	/* temporal: in reverse order */
   int i;
 
   /* initialize */
@@ -548,7 +548,7 @@ find_1pass_result(int framelen, RecogProcess *r)
 static int
 compare_backscore(TRELLIS_ATOM **x1, TRELLIS_ATOM **x2)
 {
-  return((*x1)->backscore < (*x2)->backscore);
+  return((*x2)->backscore - (*x1)->backscore);
 }
 
 /** 
