@@ -17,7 +17,7 @@
  * @author Akinobu Lee
  * @date   Fri Feb 16 15:05:43 2007
  *
- * $Revision: 1.4 $
+ * $Revision: 1.8 $
  * 
  */
 /*
@@ -56,7 +56,10 @@
 void
 jconf_set_default_values(Jconf *j)
 {
+  j->input.type				= INPUT_WAVEFORM;
   j->input.speech_input			= SP_MFCFILE;
+  j->input.device			= SP_INPUT_DEFAULT;
+  j->input.plugin_source		= -1;
   j->input.sfreq			= 16000;
   j->input.period			= 625;
   j->input.framesize			= DEF_FRAMESIZE;
@@ -94,6 +97,9 @@ jconf_set_default_values(Jconf *j)
   j->decodeopt.forced_realtime		= FALSE;
   j->decodeopt.force_realtime_flag	= FALSE;
   j->decodeopt.segment			= FALSE;
+
+  j->optsection				= JCONF_OPT_DEFAULT;
+  j->optsectioning			= TRUE;
 }
 
 /** 
