@@ -19,7 +19,7 @@
  * @author Akinobu Lee
  * @date   Wed Aug  8 15:04:28 2007
  *
- * $Revision: 1.2 $
+ * $Revision: 1.4 $
  * 
  */
 /*
@@ -57,10 +57,16 @@ j_request_pause(Recog *recog)
     recog->process_want_reload = TRUE;
     recog->process_active = FALSE;
   }
-  if (recog->jconf->input.speech_input == SP_ADINNET) {
-    /* when taking speech from adinnet client,
-       always tell the client to stop recording */
-    adin_tcpip_send_pause();
+  /* control the A/D-in module to stop recording */
+  if (recog->jconf->input.type == INPUT_WAVEFORM) {
+    if (recog->adin->ad_pause != NULL) {
+      (*(recog->adin->ad_pause))();
+    }
+  } else {
+    /* feature vector input */
+    if (recog->jconf->input.speech_input == SP_MFCMODULE) {
+      if (recog->mfcclist->func.fv_pause) recog->mfcclist->func.fv_pause();
+    }
   }
 }
 
@@ -92,10 +98,16 @@ j_request_terminate(Recog *recog)
     recog->process_want_reload = TRUE;
     recog->process_active = FALSE;
   }
-  if (recog->jconf->input.speech_input == SP_ADINNET) {
-    /* when taking speech input from adinnet client,
-       always tell the client to stop recording immediately */
-    adin_tcpip_send_terminate();
+  if (recog->jconf->input.type == INPUT_WAVEFORM) {
+    if (recog->adin->ad_terminate != NULL) {
+      /* control the A/D-in module to terminate recording imemdiately */
+      (*(recog->adin->ad_terminate))();
+    }
+  } else {
+    /* feature vector input */
+    if (recog->jconf->input.speech_input == SP_MFCMODULE) {
+      if (recog->mfcclist->func.fv_terminate) recog->mfcclist->func.fv_terminate();
+    }
   }
 }
 
@@ -120,10 +132,16 @@ j_request_resume(Recog *recog)
     recog->process_want_terminate = FALSE;
     recog->process_active = TRUE;
   }
-  if (recog->jconf->input.speech_input == SP_ADINNET) {
-    /* when taking speech from adinnet client,
-       tell the client to restart recording */
-    adin_tcpip_send_resume();
+  /* control the A/D-in module to restart recording now */
+  if (recog->jconf->input.type == INPUT_WAVEFORM) {
+    if (recog->adin->ad_resume != NULL) {
+      (*(recog->adin->ad_resume))();
+    }
+  } else {
+    /* feature vector input */
+    if (recog->jconf->input.speech_input == SP_MFCMODULE) {
+      if (recog->mfcclist->func.fv_resume) recog->mfcclist->func.fv_resume();
+    }
   }
 }
 
@@ -522,63 +540,22 @@ j_adin_init(Recog *recog)
 {
   boolean ret;
 
+  if (recog->jconf->input.type == INPUT_VECTOR) {
+    /* feature vector input */
+    if (recog->jconf->input.speech_input == SP_MFCMODULE) {
+      if (mfc_module_init(recog->mfcclist, recog) == FALSE) {
+	return FALSE;
+      }
+      ret = mfc_module_standby(recog->mfcclist);
+    } else {
+      ret = TRUE;
+    }
+    return ret;
+  }
+  
+  /* initialize A/D-in device */
   ret = adin_initialize(recog);
 
-    /* initialize A/D-in device */
-    /* create A/D-in thread here */
-#ifdef HAVE_PTHREAD
-  if (ret) {
-    if (recog->adin->enable_thread) {
-      if (adin_thread_create(recog) == FALSE) {
-	return FALSE;
-      }
-    }
-  }
-#endif
-
-  return(ret);
-}
-
-/** 
- * <EN>
- * Initialize function when using user-defined A/D-in function.
- * Almost the same with j_adin_init() except that it does not initialize
- * an input device.
- * 
- * </EN>
- * <JA>
- * 桼A/D-inؿѻνؿ. ǥХνԤʤ
- * ʳ j_adin_init() ƱǤ. 
-* 
- * </JA>
- * 
- * @param recog [in] engine instance
- * @param arg [in] argument
- * 
- * @return TRUE on success, else FALSE on error.
- * 
- * @callgraph
- * @callergraph
- * @ingroup engine
- */
-boolean
-j_adin_init_user(Recog *recog, void *arg)
-{
-  boolean ret;
-
-  ret = adin_initialize_user(recog, arg);
-
-    /* initialize A/D-in device */
-    /* create A/D-in thread here */
-#ifdef HAVE_PTHREAD
-  if (ret) {
-    if (recog->adin->enable_thread) {
-      if (adin_thread_create(recog) == FALSE) {
-	return FALSE;
-      }
-    }
-  }
-#endif
   return(ret);
 }
 
@@ -1497,4 +1474,57 @@ j_process_am_remove(Recog *recog, JCONF_AM *amconf)
   return TRUE;
 }
 
+#ifdef DEBUG_VTLN_ALPHA_TEST
+void
+vtln_alpha(Recog *recog, RecogProcess *r)
+{
+  Sentence *s;
+  float alpha, alpha_bgn, alpha_end;
+  float max_alpha;
+  LOGPROB max_score;
+  PROCESS_AM *am;
+  MFCCCalc *mfcc;
+
+  s = &(r->result.sent[0]);
+
+  max_score = LOG_ZERO;
+
+  printf("------------ begin VTLN -------------\n");
+
+  mfcc = r->am->mfcc;
+
+  alpha_bgn = mfcc->para->vtln_alpha - VTLN_RANGE;
+  alpha_end = mfcc->para->vtln_alpha + VTLN_RANGE;
+
+  for(alpha = alpha_bgn; alpha <= alpha_end; alpha += VTLN_STEP) {
+    mfcc->para->vtln_alpha = alpha;
+    if (InitFBank(mfcc->wrk, mfcc->para) == FALSE) {
+      jlog("ERROR: VTLN: InitFBank() failed\n");
+      return;
+    }
+    if (wav2mfcc(recog->speech, recog->speechlen, recog) == FALSE) {
+      jlog("ERROR: VTLN: wav2mfcc() failed\n");
+      return;
+    }
+    outprob_prepare(&(r->am->hmmwrk), mfcc->param->samplenum);
+    word_align(s->word, s->word_num, mfcc->param, s, r);
+    printf("%f: %f\n", alpha, s->align.allscore);
+    if (max_score < s->align.allscore) {
+      max_score = s->align.allscore;
+      max_alpha = alpha;
+    }
+  }
+  printf("MAX: %f: %f\n", max_alpha, max_score);
+  mfcc->para->vtln_alpha = max_alpha;
+  if (InitFBank(mfcc->wrk, mfcc->para) == FALSE) {
+    jlog("ERROR: VTLN: InitFBank() failed\n");
+    return;
+  }
+
+  printf("------------ end VTLN -------------\n");
+
+}
+#endif
+
+
 /* end of file */
