@@ -12,7 +12,7 @@
  * @author Akinobu Lee
  * @date   Thu May 12 14:14:01 2005
  *
- * $Revision: 1.5 $
+ * $Revision: 1.9 $
  * 
  */
 /*
@@ -169,16 +169,7 @@ print_engine_info(Recog *recog)
   /* print current argument setting to log */
   print_jconf_overview(jconf);
 
-  /* for backward compatibility with scoring tool (IPA99)... :-( */
-#if 0
-  if (jconf->input.speech_input == SP_RAWFILE) {
-    jlog("Speech input source: file\n\n");
-  } else if (jconf->input.speech_input == SP_MFCFILE) {
-    jlog("Speech input source: MFCC parameter file (HTK format)\n\n");
-  }
-#endif
-
-  if (jconf->input.speech_input != SP_MFCFILE) {
+  if (jconf->input.type == INPUT_WAVEFORM) {
 
     /* acoustic parameter conditions for this model */
     jlog("------------------------------------------------------------\n");
@@ -201,7 +192,7 @@ print_engine_info(Recog *recog)
 
       put_para(fp, mfcc->para);
 
-      if (jconf->input.speech_input != SP_MFCFILE) {
+      if (jconf->input.type == INPUT_WAVEFORM) {
 	jlog("    spectral subtraction = ");
 	if (mfcc->frontend.ssload_filename || mfcc->frontend.sscalc) {
 	  if (mfcc->frontend.sscalc) {
@@ -217,13 +208,23 @@ print_engine_info(Recog *recog)
 	  jlog("off\n");
 	}
       }
-      jlog("     cepstral mean norm. = ");
-      if (mfcc->para->cmn) {
+      jlog("  cepstral normalization = ");
+      if (mfcc->para->cmn || mfcc->para->cvn) {
 	if (jconf->decodeopt.realtime_flag) {
-	  jlog("real-time MAP-CMN\n");
+	  jlog("real-time MAP-");
 	} else {
-	  jlog("sentence CMN\n");
+	  jlog("sentence ");
 	}
+	if (mfcc->para->cmn) {
+	  jlog("CMN");
+	}
+	if (mfcc->para->cmn && mfcc->para->cvn) {
+	  jlog("+");
+	}
+	if (mfcc->para->cvn) {
+	  jlog("CVN");
+	}
+	jlog("\n");
       } else {
 	jlog("no\n");
       }
@@ -246,26 +247,29 @@ print_engine_info(Recog *recog)
 
       jlog("\n");
 
-      if (jconf->decodeopt.realtime_flag && mfcc->para->cmn) {
-	jlog(" MAP-CMN:\n");
-	jlog("      default cep. mean   = ");
+      if (jconf->decodeopt.realtime_flag && (mfcc->para->cmn || mfcc->para->cvn)) {
+	jlog(" MAP-");
+	if (mfcc->para->cmn) jlog("CMN");
+	if (mfcc->para->cmn && mfcc->para->cvn) jlog("+");
+	if (mfcc->para->cvn) jlog("CVN");
+	jlog(":\n");
+	jlog("      initial cep. data   = ");
 	if (mfcc->cmn.load_filename) {
 	  jlog("load from \"%s\"\n", mfcc->cmn.load_filename);
 	} else {
-	  jlog("not specified\n");
+	  jlog("none\n");
 	}
-	jlog("      initial mean weight = %6.2f\n", mfcc->cmn.map_weight);
+	jlog("      beginning data weight = %6.2f\n", mfcc->cmn.map_weight);
 	if (mfcc->cmn.update) {
-	  jlog("      initial mean update = yes, from last inputs at each input\n");
+	  jlog("    beginning data update = yes, from last inputs at each input\n");
 	} else {
-	  jlog("      initial mean update = no, use default as initial at each input\n");
+	  jlog("    beginning data update = no, use default as initial at each input\n");
 	}
 	if (mfcc->cmn.save_filename) {
-	  jlog("      save cep. mean to   = %s at end of each input\n", mfcc->cmn.save_filename);
+	  jlog("        save cep. data to = file \"%s\" at end of each input\n", mfcc->cmn.save_filename);
 	}
 	jlog("\n");
       }
-
     }
   }
 
@@ -316,9 +320,11 @@ print_engine_info(Recog *recog)
     case GPRUNE_SEL_BEAM: jlog("beam"); break;
     case GPRUNE_SEL_HEURISTIC: jlog("heuristic"); break;
     case GPRUNE_SEL_SAFE: jlog("safe"); break;
+    case GPRUNE_SEL_USER: jlog("(use plugin function)"); break;
     }
     jlog("  (-gprune)\n");
-    if (am->config->gprune_method != GPRUNE_SEL_NONE) {
+    if (am->config->gprune_method != GPRUNE_SEL_NONE
+	&& am->config->gprune_method != GPRUNE_SEL_USER) {
       jlog("  top N mixtures to calc = %d / %d  (-tmix)\n", am->config->mixnum_thres, am->hmminfo->maxcodebooksize);
     }
     if (am->config->hmm_gs_filename != NULL) {
@@ -627,19 +633,21 @@ print_engine_info(Recog *recog)
 	jlog("\t all possible words will be expanded in 2nd pass\n");
       }
     }
-    if (r->wchmm->category_tree) {
-      if (r->config->pass1.old_tree_function_flag) {
-	jlog("\t build_wchmm() used\n");
-      } else {
-	jlog("\t build_wchmm2() used\n");
-      }
+    if (r->wchmm != NULL) {
+      if (r->wchmm->category_tree) {
+	if (r->config->pass1.old_tree_function_flag) {
+	  jlog("\t build_wchmm() used\n");
+	} else {
+	  jlog("\t build_wchmm2() used\n");
+	}
 #ifdef PASS1_IWCD
 #ifdef USE_OLD_IWCD
-      jlog("\t full lcdset used\n");
+	jlog("\t full lcdset used\n");
 #else
-      jlog("\t lcdset limited by word-pair constraint\n");
+	jlog("\t lcdset limited by word-pair constraint\n");
 #endif
 #endif /* PASS1_IWCD */
+      }
     }
     if (r->config->output.progout_flag) {
       jlog("\tprogressive output on 1st pass\n");
@@ -750,21 +758,32 @@ print_engine_info(Recog *recog)
   jlog("------------------------------------------------------------\n");
   jlog("FrontEnd:\n\n");
 
-  jlog(" Speech input:\n");
-  jlog("\t    speech input source = ");
-  if (jconf->input.speech_input == SP_RAWFILE) {
-    jlog("speech file\n");
+  jlog(" Input stream:\n");
+  jlog("\t             input type = ");
+  switch(jconf->input.type) {
+  case INPUT_WAVEFORM:
+    jlog("waveform\n");
+    break;
+  case INPUT_VECTOR:
+    jlog("feature vector sequence\n");
+    break;
+  }
+  jlog("\t           input source = ");
+  if (jconf->input.plugin_source != -1) {
+    jlog("plugin\n");
+  } else if (jconf->input.speech_input == SP_RAWFILE) {
+    jlog("waveform file\n");
     jlog("\t          input filelist = ");
     if (jconf->input.inputlist_filename == NULL) {
-      jlog("(none, enter filenames from stdin)\n");
+      jlog("(none, get file name from stdin)\n");
     } else {
       jlog("%s\n", jconf->input.inputlist_filename);
     }
   } else if (jconf->input.speech_input == SP_MFCFILE) {
-    jlog("MFCC parameter file (HTK format)\n");
+    jlog("feature vector file (HTK format)\n");
     jlog("\t                filelist = ");
     if (jconf->input.inputlist_filename == NULL) {
-      jlog("(none, enter filenames from stdin)\n");
+      jlog("(none, get file name from stdin)\n");
     } else {
       jlog("%s\n", jconf->input.inputlist_filename);
     }
@@ -786,8 +805,15 @@ print_engine_info(Recog *recog)
 #endif
   } else if (jconf->input.speech_input == SP_MIC) {
     jlog("microphone\n");
+    jlog("\t    device API          = ");
+    switch(jconf->input.device) {
+    case SP_INPUT_DEFAULT: jlog("default\n"); break;
+    case SP_INPUT_ALSA: jlog("alsa\n"); break;
+    case SP_INPUT_OSS: jlog("oss\n"); break;
+    case SP_INPUT_ESD: jlog("esd\n"); break;
+    }
   }
-  if (jconf->input.speech_input != SP_MFCFILE) {
+  if (jconf->input.type == INPUT_WAVEFORM) {
     if (jconf->input.speech_input == SP_RAWFILE || jconf->input.speech_input == SP_STDIN || jconf->input.speech_input == SP_ADINNET) {
       if (jconf->input.use_ds48to16) {
 	jlog("\t          sampling freq. = assume 48000Hz, then down to %dHz\n", jconf->input.sfreq);
@@ -802,7 +828,7 @@ print_engine_info(Recog *recog)
       }
     }
   }
-  if (jconf->input.speech_input != SP_MFCFILE) {
+  if (jconf->input.type == INPUT_WAVEFORM) {
     jlog("\t         threaded A/D-in = ");
 #ifdef HAVE_PTHREAD
     if (recog->adin->enable_thread) {
@@ -819,7 +845,7 @@ print_engine_info(Recog *recog)
   } else {
     jlog("\t   zero frames stripping = off\n");
   }
-  if (jconf->input.speech_input != SP_MFCFILE) {
+  if (jconf->input.type == INPUT_WAVEFORM) {
     if (recog->adin->adin_cut_on) {
       jlog("\t         silence cutting = on\n");
       jlog("\t             level thres = %d / 32767\n", jconf->detect.level_thres);
