@@ -12,7 +12,7 @@
  * @author Akinobu Lee
  * @date   Fri May 13 15:04:34 2005
  *
- * $Revision: 1.6 $
+ * $Revision: 1.11 $
  * 
  */
 /*
@@ -45,7 +45,13 @@ void
 j_output_argument_help(FILE *fp)
 {
   Jconf *jconf;
-
+#ifdef ENABLE_PLUGIN
+  int id;
+  char buf[64];
+  PLUGIN_ENTRY *p;
+  FUNC_VOID func;
+#endif
+    
   /* load default values */
   jconf = j_jconf_new();
 
@@ -59,16 +65,42 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    (Can extract only MFCC based features from waveform)\n");
   fprintf(fp, "    [-input devname]    input source  (default = htkparam)\n");
   fprintf(fp, "         htkparam/mfcfile  HTK parameter file\n");
-  fprintf(fp, "         file/rawfile      waveform file\n");
-  fprintf(fp, "                           (format: %s)\n", SUPPORTED_WAVEFILE_FORMAT);
+  fprintf(fp, "         file/rawfile      waveform file (%s)\n", SUPPORTED_WAVEFILE_FORMAT);
 #ifdef USE_MIC
-  fprintf(fp, "         mic               microphone device\n");
+  fprintf(fp, "         mic               default microphone device\n");
+# ifdef HAS_ALSA
+  fprintf(fp, "         alsa              use ALSA interface\n");
+# endif
+# ifdef HAS_OSS
+  fprintf(fp, "         oss               use OSS interface\n");
+# endif
+# ifdef HAS_ESD
+  fprintf(fp, "         esd               use ESounD interface\n");
+# endif
 #endif
 #ifdef USE_NETAUDIO
   fprintf(fp, "         netaudio          DatLink/NetAudio server\n");
 #endif
   fprintf(fp, "         adinnet           adinnet client (TCP/IP)\n");
   fprintf(fp, "         stdin             standard input\n");
+#ifdef ENABLE_PLUGIN
+  if (global_plugin_list) {
+    if ((id = plugin_get_id("adin_get_optname")) >= 0) {
+      for(p=global_plugin_list[id];p;p=p->next) {
+	func = (FUNC_VOID) p->func;
+	(*func)(buf, (int)64);
+	fprintf(fp, "         %-18s(adin plugin #%d)\n", buf, p->source_id);
+      }
+    }
+    if ((id = plugin_get_id("fvin_get_optname")) >= 0) {
+      for(p=global_plugin_list[id];p;p=p->next) {
+	func = (FUNC_VOID) p->func;
+	(*func)(buf, (int)64);
+	fprintf(fp, "         %-18s(feature vector input plugin #%d)\n", buf, p->source_id);
+      }
+    }
+  }
+#endif
   fprintf(fp, "    [-filelist file]    filename of input file list\n");
 #ifdef USE_NETAUDIO
   fprintf(fp, "    [-NA host:unit]     get audio from NetAudio server at host:unit\n");
@@ -125,6 +157,8 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-LM]               start a new language model instance\n");
   fprintf(fp, "    [-SR]               start a new recognizer (search) instance\n");
   fprintf(fp, "    [-AM_GMM]           start an AM feature instance for GMM\n");
+  fprintf(fp, "    [-GLOBAL]           start a global section\n");
+  fprintf(fp, "    [-nosectioncheck]   disable option location check\n");
   fprintf(fp, "\n--- Acoustic Model Options (-AM) ---------------------------------\n");
 
   fprintf(fp, "\n Acoustic analysis:\n");
@@ -155,6 +189,8 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-cmnsave file]     save CMN param to file after each input\n");
   fprintf(fp, "    [-cmnnoupdate]      not update CMN param while recog. (use with -cmnload)\n");
   fprintf(fp, "    [-cmnmapweight]     weight value of initial cm for MAP-CMN (%6.2f)\n", jconf->am_root->analysis.cmn_map_weight);
+  fprintf(fp, "    [-cvn]              cepstral variance normalisation       (%s)\n", jconf->amnow->analysis.para.cvn ? "on" : "off");
+  fprintf(fp, "    [-vtln alpha lowcut hicut] enable VTLN (1.0 to disable)   (%f)\n", jconf->am_root->analysis.para_default.vtln_alpha);
 
   fprintf(fp, "\n Acoustic Model:\n");
   fprintf(fp, "    -h hmmdefsfile      HMM definition file name\n");
@@ -187,6 +223,17 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "             beam          beam pruning\n");
 #endif
   fprintf(fp, "             none          no pruning (default for non tmix models)\n");
+#ifdef ENABLE_PLUGIN
+  if (global_plugin_list) {
+    if ((id = plugin_get_id("calcmix_get_optname")) >= 0) {
+      for(p=global_plugin_list[id];p;p=p->next) {
+	func = (FUNC_VOID) p->func;
+	(*func)(buf, (int)64);
+	fprintf(fp, "             %-14s(calculation plugin #%d)\n", buf, p->source_id);
+      }
+    }
+  }
+#endif
   fprintf(fp, "    [-tmix gaussnum]    Gaussian num threshold per mixture for pruning (%d)\n", jconf->am_root->mixnum_thres);
   fprintf(fp, "    [-gshmm hmmdefs]    monophone hmmdefs for GS\n");
   fprintf(fp, "    [-gsnum N]          N-best state will be selected        (%d)\n", jconf->am_root->gs_statenum);
