@@ -1,7 +1,5 @@
 /**
  * @file   strip.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 00:30:38 2005
  *
  * <JA>
  * @brief  ǡ饼ץ
@@ -10,13 +8,16 @@
  * @brief  Strip zero samples from speech data
  * </EN>
  *
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 00:30:38 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -62,7 +63,7 @@ strip_zero(SP16 a[], int len)
 	  }
         } else {
           /* deleted (leave uncopied) */
-	  /*j_printf("Warning: sample %d-%d is invalid, stripped\n", bgn, src-1);*/
+	  jlog("Warning: strip: sample %d-%d is invalid, stripped\n", bgn, src-1);
         }
       }
       a[dst++] = a[src];
@@ -78,7 +79,7 @@ strip_zero(SP16 a[], int len)
       }
     } else {
       /* deleted (leave uncopied) */
-      /*j_printf("Warning: sample %d-%d is invalid, stripped\n", bgn, src-1);*/
+      jlog("Warning: strip: sample %d-%d is invalid, stripped\n", bgn, src-1);
     }
   }
   
