@@ -1,7 +1,5 @@
 /**
  * @file   rdwt.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 07:09:25 2005
  * 
  * <JA>
  * @brief  TCP/IPץ̿Τ٥ؿ
@@ -11,13 +9,16 @@
  * @brief  Low level functions for TCP/IP inter-process communication
  * </EN>
  * 
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 07:09:25 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -52,12 +53,11 @@ rd(int fd, char *data, int *len, int maxlen)
        read(fd,(char *)len,sizeof(int))
 #endif
        ) != sizeof(int)) {
-    /*j_printerr( "failed to read num\n");*/
+    /*jlog( "failed to read num\n");*/
     return(-1);
   }
   if (*len > maxlen) {
-    j_printerr( "transfer data length exceeded: %d (>%d)\n",
-	    len, maxlen);
+    jlog("Error: rdwt: transfer data length exceeded: %d (>%d)\n", len, maxlen);
     return(-1);
   }
   while (count<(*len)){
@@ -71,7 +71,7 @@ rd(int fd, char *data, int *len, int maxlen)
 	 read(fd,data+count,tmplen)
 #endif
 	 ) < 0) {
-      j_printerr( "failed to read data at %d / %d\n",count, len);
+      jlog("Error: rdwt: failed to read data at %d / %d\n",count, len);
       return(count);
     }
     count += tmpbytes;
@@ -101,7 +101,7 @@ wt(int fd, char *data, int len)
        write(fd,(char *)&len,sizeof(int))
 #endif
        ) != sizeof(int)) {
-    /*j_printerr( "failed to write num\n");*/
+    /*jlog( "failed to write num\n");*/
     return(-1);
   }
   if (len > 0) {
@@ -112,7 +112,7 @@ wt(int fd, char *data, int len)
 	 write(fd,data,len)
 #endif
 	 ) < 0) {
-      j_printerr( "failed to write data (%d bytes)\n",len);
+      jlog("Error: rdwt: failed to write data (%d bytes)\n",len);
       return(-1);
     }
   } else {
